package javax.activation;

import java.util.Iterator;
import java.util.Map;
import java.util.Locale;
import java.net.URL;
import java.io.InputStream;
import java.io.IOException;
import java.util.List;
import java.io.File;
import com.sun.activation.registries.LogSupport;
import java.util.ArrayList;
import com.sun.activation.registries.MailcapFile;

public class MailcapCommandMap extends CommandMap
{
    private MailcapFile[] DB;
    private static final int PROG = 0;
    
    public MailcapCommandMap() {
        List<MailcapFile> dbv = new ArrayList(5);
        MailcapFile mf = null;
        dbv.add(null);
        LogSupport.log("MailcapCommandMap: load HOME");
        try {
            String user_home = System.getProperty("user.home");
            if (user_home != null) {
                String path = user_home + File.separator + ".mailcap";
                mf = this.loadFile(path);
                if (mf != null) {
                    dbv.add(mf);
                }
            }
        }
        catch (SecurityException ex) {}
        LogSupport.log("MailcapCommandMap: load SYS");
        try {
            String system_mailcap = System.getProperty("java.home") + File.separator + "lib" + File.separator + "mailcap";
            mf = this.loadFile(system_mailcap);
            if (mf != null) {
                dbv.add(mf);
            }
        }
        catch (SecurityException ex2) {}
        LogSupport.log("MailcapCommandMap: load JAR");
        this.loadAllResources(dbv, "META-INF/mailcap");
        LogSupport.log("MailcapCommandMap: load DEF");
        mf = this.loadResource("/META-INF/mailcap.default");
        if (mf != null) {
            dbv.add(mf);
        }
        this.DB = new MailcapFile[dbv.size()];
        this.DB = (MailcapFile[])(MailcapFile[])dbv.toArray(this.DB);
    }
    
    private MailcapFile loadResource(String name) {
        InputStream clis = null;
        try {
            clis = SecuritySupport.getResourceAsStream(this.getClass(), name);
            if (clis != null) {
                MailcapFile mf = new MailcapFile(clis);
                if (LogSupport.isLoggable()) {
                    LogSupport.log("MailcapCommandMap: successfully loaded mailcap file: " + name);
                }
                return mf;
            }
            else {
                if (LogSupport.isLoggable()) {
                    LogSupport.log("MailcapCommandMap: not loading mailcap file: " + name);
                }
                return null;
            }
        }
        catch (IOException e) {
            if (LogSupport.isLoggable()) {
                LogSupport.log("MailcapCommandMap: can't load " + name, e);
            }
        }
        catch (SecurityException sex) {
            if (LogSupport.isLoggable()) {
                LogSupport.log("MailcapCommandMap: can't load " + name, sex);
            }
        }
        finally {
            try {
                if (clis != null) {
                    clis.close();
                }
            }
            catch (IOException ex) {}
        }
        return null;
    }
    
    private void loadAllResources(List v, String name) {
        boolean anyLoaded = false;
        try {
            ClassLoader cld = null;
            cld = SecuritySupport.getContextClassLoader();
            if (cld == null) {
                cld = this.getClass().getClassLoader();
            }
            URL[] urls;
            if (cld != null) {
                urls = SecuritySupport.getResources(cld, name);
            }
            else {
                urls = SecuritySupport.getSystemResources(name);
            }
            if (urls != null) {
                if (LogSupport.isLoggable()) {
                    LogSupport.log("MailcapCommandMap: getResources");
                }
                int i = 0;
                while (i < urls.length) {
                    URL url = urls[i];
                    InputStream clis = null;
                    Block_11: {
                        if (LogSupport.isLoggable()) {
                            continue;
                        }
                        LogSupport.log("MailcapCommandMap: URL " + url);
                        try {
                            clis = SecuritySupport.openStream(url);
                            if (clis != null) {
                                v.add(new MailcapFile(clis));
                                anyLoaded = true;
                                if (LogSupport.isLoggable()) {
                                    LogSupport.log("MailcapCommandMap: successfully loaded mailcap file from URL: " + url);
                                }
                            }
                            else if (LogSupport.isLoggable()) {
                                LogSupport.log("MailcapCommandMap: not loading mailcap file from URL: " + url);
                            }
                        }
                        catch (IOException ioex) {
                            if (LogSupport.isLoggable()) {
                                LogSupport.log("MailcapCommandMap: can't load " + url, ioex);
                            }
                        }
                        catch (SecurityException sex) {
                            if (LogSupport.isLoggable()) {
                                LogSupport.log("MailcapCommandMap: can't load " + url, sex);
                            }
                        }
                        finally {
                            try {
                                if (clis != null) {
                                    clis.close();
                                }
                            }
                            catch (IOException ex2) {}
                        }
                        ++i;
                    }
                }
            }
        }
        catch (Exception ex) {
            if (LogSupport.isLoggable()) {
                LogSupport.log("MailcapCommandMap: can't load " + name, ex);
            }
        }
        if (!anyLoaded) {
            if (LogSupport.isLoggable()) {
                LogSupport.log("MailcapCommandMap: !anyLoaded");
            }
            MailcapFile mf = this.loadResource("/" + name);
            if (mf != null) {
                v.add(mf);
            }
        }
    }
    
    private MailcapFile loadFile(String name) {
        MailcapFile mtf = null;
        try {
            mtf = new MailcapFile(name);
        }
        catch (IOException ex) {}
        return mtf;
    }
    
    public MailcapCommandMap(String fileName) throws IOException {
        this();
        if (LogSupport.isLoggable()) {
            LogSupport.log("MailcapCommandMap: load PROG from " + fileName);
        }
        if (this.DB[0] == null) {
            this.DB[0] = new MailcapFile(fileName);
        }
    }
    
    public MailcapCommandMap(InputStream is) {
        this();
        LogSupport.log("MailcapCommandMap: load PROG");
        if (this.DB[0] == null) {
            try {
                this.DB[0] = new MailcapFile(is);
            }
            catch (IOException ex) {}
        }
    }
    
    @Override
    public synchronized CommandInfo[] getPreferredCommands(String mimeType) {
        List<Object> cmdList = new ArrayList();
        if (mimeType != null) {
            mimeType = mimeType.toLowerCase(Locale.ENGLISH);
        }
        for (int i = 0; i < this.DB.length; ++i) {
            if (this.DB[i] != null) {
                Map<Object, Object> cmdMap = this.DB[i].getMailcapList(mimeType);
                if (cmdMap != null) {
                    this.appendPrefCmdsToList(cmdMap, cmdList);
                }
            }
        }
        for (int i = 0; i < this.DB.length; ++i) {
            if (this.DB[i] != null) {
                Map<Object, Object> cmdMap = this.DB[i].getMailcapFallbackList(mimeType);
                if (cmdMap != null) {
                    this.appendPrefCmdsToList(cmdMap, cmdList);
                }
            }
        }
        CommandInfo[] cmdInfos = new CommandInfo[cmdList.size()];
        cmdInfos = (CommandInfo[])(CommandInfo[])cmdList.toArray(cmdInfos);
        return cmdInfos;
    }
    
    private void appendPrefCmdsToList(Map cmdHash, List cmdList) {
        for (String verb : cmdHash.keySet()) {
            if (!this.checkForVerb(cmdList, verb)) {
                List<Object> cmdList2 = (List)cmdHash.get(verb);
                String className = (String)cmdList2.get(0);
                cmdList.add(new CommandInfo(verb, className));
            }
            else {
                continue;
            }
        }
    }
    
    private boolean checkForVerb(List cmdList, String verb) {
        Iterator<Object> ee = cmdList.iterator();
        while (ee.hasNext()) {
            String enum_verb = ((CommandInfo)ee.next()).getCommandName();
            if (enum_verb.equals(verb)) {
                return true;
            }
            continue;
        }
        return false;
    }
    
    @Override
    public synchronized CommandInfo[] getAllCommands(String mimeType) {
        List<Object> cmdList = new ArrayList();
        if (mimeType != null) {
            mimeType = mimeType.toLowerCase(Locale.ENGLISH);
        }
        for (int i = 0; i < this.DB.length; ++i) {
            if (this.DB[i] != null) {
                Map<Object, Object> cmdMap = this.DB[i].getMailcapList(mimeType);
                if (cmdMap != null) {
                    this.appendCmdsToList(cmdMap, cmdList);
                }
            }
        }
        for (int i = 0; i < this.DB.length; ++i) {
            if (this.DB[i] != null) {
                Map<Object, Object> cmdMap = this.DB[i].getMailcapFallbackList(mimeType);
                if (cmdMap != null) {
                    this.appendCmdsToList(cmdMap, cmdList);
                }
            }
        }
        CommandInfo[] cmdInfos = new CommandInfo[cmdList.size()];
        cmdInfos = (CommandInfo[])(CommandInfo[])cmdList.toArray(cmdInfos);
        return cmdInfos;
    }
    
    private void appendCmdsToList(Map typeHash, List cmdList) {
        for (String verb : typeHash.keySet()) {
            List<Object> cmdList2 = (List)typeHash.get(verb);
            for (String cmd : cmdList2) {
                cmdList.add(new CommandInfo(verb, cmd));
            }
        }
    }
    
    @Override
    public synchronized CommandInfo getCommand(String mimeType, String cmdName) {
        if (mimeType != null) {
            mimeType = mimeType.toLowerCase(Locale.ENGLISH);
        }
        for (int i = 0; i < this.DB.length; ++i) {
            if (this.DB[i] != null) {
                Map<Object, Object> cmdMap = this.DB[i].getMailcapList(mimeType);
                if (cmdMap != null) {
                    List<Object> v = (List)cmdMap.get(cmdName);
                    if (v != null) {
                        String cmdClassName = (String)v.get(0);
                        if (cmdClassName != null) {
                            return new CommandInfo(cmdName, cmdClassName);
                        }
                    }
                }
            }
        }
        for (int i = 0; i < this.DB.length; ++i) {
            if (this.DB[i] != null) {
                Map<Object, Object> cmdMap = this.DB[i].getMailcapFallbackList(mimeType);
                if (cmdMap != null) {
                    List<Object> v = (List)cmdMap.get(cmdName);
                    if (v != null) {
                        String cmdClassName = (String)v.get(0);
                        if (cmdClassName != null) {
                            return new CommandInfo(cmdName, cmdClassName);
                        }
                    }
                }
            }
        }
        return null;
    }
    
    public synchronized void addMailcap(String mail_cap) {
        LogSupport.log("MailcapCommandMap: add to PROG");
        if (this.DB[0] == null) {
            this.DB[0] = new MailcapFile();
        }
        this.DB[0].appendToMailcap(mail_cap);
    }
    
    @Override
    public synchronized DataContentHandler createDataContentHandler(String mimeType) {
        if (LogSupport.isLoggable()) {
            LogSupport.log("MailcapCommandMap: createDataContentHandler for " + mimeType);
        }
        if (mimeType != null) {
            mimeType = mimeType.toLowerCase(Locale.ENGLISH);
        }
        for (int i = 0; i < this.DB.length; ++i) {
            if (this.DB[i] != null) {
                if (LogSupport.isLoggable()) {
                    LogSupport.log("  search DB #" + i);
                }
                Map<Object, Object> cmdMap = this.DB[i].getMailcapList(mimeType);
                if (cmdMap != null) {
                    List<Object> v = (List)cmdMap.get("content-handler");
                    if (v != null) {
                        String name = (String)v.get(0);
                        DataContentHandler dch = this.getDataContentHandler(name);
                        if (dch != null) {
                            return dch;
                        }
                    }
                }
            }
        }
        for (int i = 0; i < this.DB.length; ++i) {
            if (this.DB[i] != null) {
                if (LogSupport.isLoggable()) {
                    LogSupport.log("  search fallback DB #" + i);
                }
                Map<Object, Object> cmdMap = this.DB[i].getMailcapFallbackList(mimeType);
                if (cmdMap != null) {
                    List<Object> v = (List)cmdMap.get("content-handler");
                    if (v != null) {
                        String name = (String)v.get(0);
                        DataContentHandler dch = this.getDataContentHandler(name);
                        if (dch != null) {
                            return dch;
                        }
                    }
                }
            }
        }
        return null;
    }
    
    private DataContentHandler getDataContentHandler(String name) {
        if (LogSupport.isLoggable()) {
            LogSupport.log("    got content-handler");
        }
        if (LogSupport.isLoggable()) {
            LogSupport.log("      class " + name);
        }
        try {
            ClassLoader cld = null;
            cld = SecuritySupport.getContextClassLoader();
            if (cld == null) {
                cld = this.getClass().getClassLoader();
            }
            Class<Object> cl = null;
            try {
                cl = cld.loadClass(name);
            }
            catch (Exception ex) {
                cl = Class.forName(name);
            }
            if (cl != null) {
                return (DataContentHandler)cl.newInstance();
            }
        }
        catch (IllegalAccessException e) {
            if (LogSupport.isLoggable()) {
                LogSupport.log("Can't load DCH " + name, e);
            }
        }
        catch (ClassNotFoundException e2) {
            if (LogSupport.isLoggable()) {
                LogSupport.log("Can't load DCH " + name, e2);
            }
        }
        catch (InstantiationException e3) {
            if (LogSupport.isLoggable()) {
                LogSupport.log("Can't load DCH " + name, e3);
            }
        }
        return null;
    }
    
    @Override
    public synchronized String[] getMimeTypes() {
        List<Object> mtList = new ArrayList();
        for (int i = 0; i < this.DB.length; ++i) {
            if (this.DB[i] != null) {
                String[] ts = this.DB[i].getMimeTypes();
                if (ts != null) {
                    for (int j = 0; j < ts.length; ++j) {
                        if (!mtList.contains(ts[j])) {
                            mtList.add(ts[j]);
                        }
                    }
                }
            }
        }
        String[] mts = new String[mtList.size()];
        mts = (String[])(String[])mtList.toArray(mts);
        return mts;
    }
    
    public synchronized String[] getNativeCommands(String mimeType) {
        List<Object> cmdList = new ArrayList();
        if (mimeType != null) {
            mimeType = mimeType.toLowerCase(Locale.ENGLISH);
        }
        for (int i = 0; i < this.DB.length; ++i) {
            if (this.DB[i] != null) {
                String[] cmds = this.DB[i].getNativeCommands(mimeType);
                if (cmds != null) {
                    for (int j = 0; j < cmds.length; ++j) {
                        if (!cmdList.contains(cmds[j])) {
                            cmdList.add(cmds[j]);
                        }
                    }
                }
            }
        }
        String[] cmds2 = new String[cmdList.size()];
        cmds2 = (String[])(String[])cmdList.toArray(cmds2);
        return cmds2;
    }
}
